###-----------------------------------------------------------------------------
### example: build_mcu_fpga_ip.tcl
### A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
###
### Contributors
###
### David Flynn (d.w.flynn@soton.ac.uk)
###
### Copyright  2022, SoC Labs (www.soclabs.org)
###-----------------------------------------------------------------------------
#
# developed & tested using vivado_version 2021.1
#
# usage:
# vivado -mode tcl -source scripts/build_mcu_fpga_ip.tcl

#
# STEP#0: define output directory area.
#

set outputDir ./vivado/built_mcu_fpga
file mkdir $outputDir
#
# STEP#1: setup design sources and constraints
#

# local search path for configurations
set search_path ../verilog

set cortexm0_vlog    ../../../../../../arm-AAA-ip/Cortex-M0/AT510-BU-00000-r0p0-03rel3/logical
source scripts/rtl_source_cm0.tcl

set search_path [ concat $search_path $cortexm0_vlog/cortexm0_integration/verilog ]
read_verilog  [ glob $cortexm0_vlog/cortexm0_integration/verilog/*.v ]
read_verilog  [ glob $cortexm0_vlog/models/cells/*.v ]

# Arm unmodified CMSDK RTL
set cmsdk_vlog    ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0
source scripts/rtl_source_cmsdk.tcl

set search_path [ concat $search_path $cmsdk_vlog/logical/models/memories ]
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_memory_models_defs.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_ram.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_sram.v

# ADP, FT1248 and streamio IP
source scripts/rtl_source_soclabs_ip.tcl

## FPGA-specific pads
#source scripts/rtl_source_fpga_ip.tcl

# soclabs modified mcu system 
set soc_vlog ../verilog
read_verilog  $soc_vlog/cmsdk_mcu_defs.v
read_verilog  $soc_vlog/ahb_bootrom.v
read_verilog  $soc_vlog/bootrom.v
read_verilog  $soc_vlog/cmsdk_ahb_cs_rom_table.v
read_verilog  $soc_vlog/cmsdk_apb_usrt.v
read_verilog  $soc_vlog/cmsdk_mcu_addr_decode.v
read_verilog  $soc_vlog/cmsdk_mcu_clkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_pin_mux.v
read_verilog  $soc_vlog/cmsdk_mcu_stclkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_sysctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_system.v
read_verilog  $soc_vlog/cmsdk_mcu_chip.v

# FPGA specific timing constraints
#read_xdc target_fpga/fpga_timing.xdc

## FPGA board specific pin constraints
#read_xdc target_fpga/fpga_pinmap.xdc

#
# STEP#2: run synthesis, report utilization and timing estimates, write checkpoint design
#

update_compile_order -fileset sources_1

set mculib_ip   ../../../../../MCULIB

ipx::package_project -root_dir $mculib_ip -vendor soclabs.org -library user -taxonomy /UserIP -import_files -set_current false -force

ipx::unload_core  $mculib_ip/component.xml
ipx::edit_ip_in_project -upgrade true -name tmp_edit_project -directory  $mculib_ip  $mculib_ip/component.xml

update_compile_order -fileset sources_1
set_property ipi_drc {ignore_freq_hz true} [ipx::current_core]
ipx::merge_project_changes files [ipx::current_core]

set_property core_revision 2 [ipx::current_core]
ipx::update_source_project_archive -component [ipx::current_core]
ipx::create_xgui_files [ipx::current_core]
ipx::update_checksums [ipx::current_core]
ipx::check_integrity [ipx::current_core]

ipx::save_core [ipx::current_core]
ipx::check_integrity -quiet -xrt [ipx::current_core]
ipx::archive_core  $mculib_ip/soclabs.org_user_cmsdk_mcu_chip_1.0.zip [ipx::current_core]
ipx::move_temp_component_back -component [ipx::current_core]
close_project -delete

#set_property  ip_repo_paths  {/home/dwf1m12/soclabs/fpga/vivado/pynq/ip_repo /home/dwf1m12/soclabs_git/soclabs-cortexm0-mcu/MCULIB} [current_project]
set_property  ip_repo_paths  {/home/dwf1m12/soclabs/fpga/vivado/pynq/ip_repo ../../../../../MCULIB} [current_project]
update_ip_catalog
close_project

#create_project project_2 /home/dwf1m12/soclabs_git/soclabs-cortexm0-mcu/Cortex-M0/soclabs_demo/systems/cortex_m0_mcu/fpga_imp/project_2 -part xczu7ev-ffvc1156-2-e
#set_property board_part xilinx.com:zcu104:part0:1.1 [current_project]
#set_property coreContainer.enable 1 [current_project]
#set_property  ip_repo_paths  /home/dwf1m12/soclabs/fpga/vivado/pynq/ip_repo [current_project]
#update_ip_catalog
